#!/usr/bin/env python

from retry import retry
import requests
import logging
import json

logging.getLogger("requests").setLevel(logging.WARNING)
log = logging.getLogger(__name__)


class Vault():

    @retry(tries=3)
    def make_request(self, method, resource, **args):
        '''
        Make a request to Vault
        '''

        def __init__(self, vault_server=None, token=None, **kwargs):
            self.token = token
            self.vault_server = vault_server

        try:
            url = "{0}/v1/{1}".format(self.vault_server, resource)
            headers = {'X-Vault-Token': self.token, 'Content-Type': 'application/json'}
            data = json.loads(requests.request(method, url, headers=headers, **args).content)
            if 'errors' in data and len(data['errors']) > 0:
                raise Exception("%s" % ','.join(data['errors']))
            else:
                return data
        except Exception as e:
            log.error('Failed to make request! %s: %s' % (type(e).__name__, e))
            raise

    def login(self, provider, login_payload, **kwargs):
        '''
        Generates Vault token from login request
        '''
        try:
            login_data = self.make_request('POST', "auth/%s/login" %
                                           (provider), data=json.dumps(login_payload))
            self.token = login_data['auth']['client_token']
        except Exception as e:
            log.error('Failed to get token! %s: %s' % (type(e).__name__, e))
            raise

    def read(self, path, key=None, **kwargs):
        '''
        Return the value of key at path in vault, or entire secret
        '''
        try:
            data = self.make_request('GET', path)['data']
            if key is not None:
                return data[key]
            return data
        except Exception as e:
            log.error('Failed to read secret! %s: %s' % (type(e).__name__, e))
            raise



#!/usr/bin/env python

import requests
import logging


class Instance:

    @staticmethod
    def get_cloud_provider(**kwargs):
        provider = ""
        if requests.get("http://169.254.169.254/").content.decode().split("\n")[1] == "computeMetadata/":
            provider = "gcp"
        else:
            provider = "aws"
        return provider



#!/usr/bin/env python

import sys
import logging
import base64
import json
import requests
import botocore.session
from botocore.awsrequest import create_request_object
from retry import retry
from cloudutils.vault import Vault
import json

log = logging.getLogger(__name__)


class VaultEC2(Vault):

    def __init__(self, vault_server, instance_object,  **kwargs):
        self.vault_server = vault_server
        self.token = None
        self.login(provider='aws', login_payload=self.generate_vault_auth_payload(
            instance_object=instance_object))

    def headers_to_go_style(self, headers, **kwargs):
        retval = {}
        for k, v in headers.items():
            if type(v) == bytes:
                retval[k] = [v.decode()]
            else:
                retval[k] = [v]
        return retval

    @retry(tries=3)
    def generate_vault_auth_payload(self, instance_object, **kwargs):
        '''
        Generate request payload for Vault login
        '''
        try:
            session = botocore.session.get_session()
            client = session.create_client('sts')
            endpoint = client._endpoint
            operation_model = client._service_model.operation_model('GetCallerIdentity')
            request_dict = client._convert_to_request_dict({}, operation_model)
            request_dict['headers']['X-Vault-AWS-IAM-Server-ID'] = self.vault_server
            request = endpoint.create_request(request_dict, operation_model)

            headers = json.dumps(self.headers_to_go_style(dict(request.headers)))

            role_with_environment = ["development", "staging"]

            if instance_object.tags['environment'] in role_with_environment:
                role = "{service}-{environment}-{region}".format(
                    service=instance_object.tags['service'],
                    environment=instance_object.tags['environment'],
                    region=instance_object.region,
                )
            else:
                role = "{service}-{region}".format(
                    service=instance_object.tags['service'],
                    region=instance_object.region,
                )

            return {
                'role': role,
                'iam_http_request_method': request.method,
                'iam_request_url':         base64.b64encode(bytes(request.url, 'utf-8')).decode(),
                'iam_request_body':        base64.b64encode(bytes(request.body, 'utf-8')).decode(),
                'iam_request_headers':     base64.b64encode(bytes(headers, 'utf-8')).decode(),
            }
        except Exception as e:
            log.error('Failed to generate auth payload! %s: %s' % (type(e).__name__, e))
            raise



#!/usr/bin/env python

import requests
import logging
import urllib
from cloudutils.vault import Vault

log = logging.getLogger(__name__)


class VaultGCE(Vault):

    GCP_METADATA_HEADERS = {
        "Metadata-Flavor": "Google"
    }

    GCP_METADATA_URL = 'http://169.254.169.254/computeMetadata/v1'

    def __init__(self,  vault_server, instance_object, **kwargs):
        self.vault_server = vault_server
        self.token = None
        self.login(provider='gcp', login_payload=self.generate_vault_auth_payload(
            instance_object=instance_object))

    def generate_vault_auth_payload(self,  instance_object, **kwargs):
        try:
            role = "%s-%s" % (instance_object.tags['service'], instance_object.zone)
            url_params = [
                "audience=%s" % (urllib.quote("http://vault/%s" % (role), safe="")),
                "format=full"
            ]

            jwt = requests.get("%s/instance/service-accounts/default/identity?%s" % (self.GCP_METADATA_URL,
                                                                                     "&".join(url_params)), headers=self.GCP_METADATA_HEADERS).content

            return {"role": role, "jwt": jwt}
        except Exception as e:
            log.error('Failed to generate auth payload! %s: %s' % (type(e).__name__, e))
            raise



#!/usr/bin/env python

import requests
import logging
from cloudutils.instance import Instance


class InstanceGCE(Instance):

    GCP_METADATA_HEADERS = {
        "Metadata-Flavor": "Google"
    }

    GCP_METADATA_URL = 'http://169.254.169.254/computeMetadata/v1'

    FILTER_TAGS = ['', 'block-project-ssh-keys', 'sshKeys']

    def __init__(self, **kwargs):
        self.is_gce = True
        self.provider = "gcp"
        self.project = self.get_project()
        self.region = self.get_region()
        self.zone = self.get_zone()

    def load_tags(self, **kwargs):
        self.tags = self.get_tags()
        return self

    def get_project(self, **kwargs):
        return requests.get("%s/project/project-id" % (self.GCP_METADATA_URL),
                            headers=self.GCP_METADATA_HEADERS).content.decode()

    def get_zone(self, **kwargs):
        zone = requests.get("%s/instance/zone" % (self.GCP_METADATA_URL),
                            headers=self.GCP_METADATA_HEADERS).content.decode().split("/")[3]

        return zone

    def get_region(self, **kwargs):
        return self.get_zone()[:-2]

    def datacenter(self, **kwargs):
        if "datacenter" in self.tags:
            return self.tags['datacenter']
        else:
            return None

    def get_tags(self, **kwargs):
        tags = {}

        instance_tags = requests.get("%s/instance/attributes/" % (self.GCP_METADATA_URL),
                                     headers=self.GCP_METADATA_HEADERS).content.decode().split("\n")
        for tag in instance_tags:
            if tag not in self.FILTER_TAGS:
                value = requests.get("%s/instance/attributes/%s" %
                                     (self.GCP_METADATA_URL, tag), headers=self.GCP_METADATA_HEADERS).content.decode()
                if "," in value:
                    tags[tag] = value.split(",")
                else:
                    tags[tag] = self.autoconvert(value)

        return tags

    def boolify(self, s, **kwargs):
        if s == 'True':
            return True
        if s == 'False':
            return False
        raise ValueError("not bool")

    def autoconvert(self, s, **kwargs):
        for fn in (self.boolify, int, float):
            try:
                return fn(s)
            except ValueError:
                pass
        return s



#!/usr/bin/env python

import requests
import logging
from cloudutils.instance import Instance

try:
    import boto3
    from botocore.credentials import InstanceMetadataProvider, InstanceMetadataFetcher
    from botocore.config import Config
except ImportError:
    print("not on AWS")  # need to handle this with an exception


class InstanceEC2(Instance):

    AWS_METADATA_HEADERS = {
    }

    AWS_METADATA_URL = "http://169.254.169.254/latest/meta-data"

    AWS_CONFIG = Config(
        retries=dict(
            max_attempts=10
        )
    )

    def __init__(self, **kwargs):
        self.is_aws = True
        self.provider = "aws"
        self.project = None
        self.zone = self.get_zone()
        self.region = self.get_region()
        self.instance_id = self.get_instance_id()

    def load_tags(self, **kwargs):
        self.tags = self.get_tags()
        return self

    def get_instance_id(self, **kwargs):
        instance_id = requests.get("%s/instance-id" % (self.AWS_METADATA_URL)).content.decode()
        return instance_id

    def get_zone(self, **kwargs):
        zone = requests.get(
            "%s/placement/availability-zone" % (self.AWS_METADATA_URL)).content.decode()
        return zone

    def get_region(self, **kwargs):
        return self.get_zone()[:-1]

    def datacenter(self, **kwargs):
        if "datacenter" in self.tags:
            return self.tags['datacenter']
        else:
            return None

    def get_tags(self, *kwargs):
        tags = {}

        ec2 = boto3.resource('ec2', region_name=self.region, config=self.AWS_CONFIG)
        instance = ec2.Instance(self.instance_id)
        for tag in instance.tags:
            if ":" in tag['Key']:
                True
            elif "," in tag['Value']:
                tags[tag['Key']] = tag['Value'].split(",")
            else:
                tags[tag['Key']] = self.autoconvert(tag['Value'])

        return tags

    def boolify(self, s, **kwargs):
        if s == 'True':
            return True
        if s == 'False':
            return False
        raise ValueError("not bool")

    def autoconvert(self, s, **kwargs):
        for fn in (self.boolify, int, float):
            try:
                return fn(s)
            except ValueError:
                pass
        return s
